<?php

namespace Teknisa\SM\Factory;
use Doctrine\ORM\EntityManager;

class IntegracaoCpFactory {
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Retorna os dados da negociação de um contrato
    **/
    public function getDadosNegociacao(array $params, $idelementos, $cdservorca) {
        $sql = "SELECT DISTINCT NEGOCIACAO.NRSEQCONTRATO,
                   NEGOCIACAO.NRTIPOORCAMENTO,
                   NEGOCIACAO.NRNEGOCIACAOCONTRATO,
                   NEGOCIACAO.CDFILIALFATURA,
                   NEGOCIACAO.NRPARCFATURA,
                   NEGOCIACAO.CDTIPOOPERFATURA,
                   NEGOCIACAO.CDSERIENF,
                   NEGOCIACAO.CDEMPRESA,
                   NEGOCIACAO.CDINSCESTA,
                   NEGOCIACAO.CDINSCMUNI,
                   NEGOCIACAO.IDTPFATURA
                FROM FCT_NEGOCIACAOCONTRATO NEGOCIACAO
                WHERE NEGOCIACAO.NRORG         = :NRORG
                  AND NEGOCIACAO.NRSEQCONTRATO = :NRSEQCONTRATO
                  AND NEGOCIACAO.NRNEGOCIACAOCONTRATO IN(SELECT NRNEGOCIACAOCONTRATO 
                                                         FROM FCT_ITNEGOCIACAOCONTRATO ITNEGO
                                                         JOIN FCT_ITEMCONTRATO ITEM
                                                            ON ITNEGO.NRITEMOCONTRATO = ITEM.NRITEMOCONTRATO
                                                            AND ITNEGO.NRORG          = ITEM.NRORG
                                                         WHERE ITEM.CDSERVORCA IN($cdservorca)
                                                            AND ITEM.IDELEMENTO IN('$idelementos')
                                                            AND ITEM.NRORG         = :NRORG
                                                            AND ITEM.NRSEQCONTRATO = :NRSEQCONTRATO)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * busca os dados de vencimento dos itens de parcelamento
    **/
    public function getDadosVencimentoItEscopo(array $params) {
        $sql = "SELECT FAT.QTDIASFATUVENC, 
                  FAT.NRDIACALCVENC, 
                  FAT.IDTPCALCVENC, 
                  FAT.IDMESCALCVENC,
                  FAT.QTDIASAPURACAO,
                  30 AS QTDIASCOMPETENCIA
                FROM FCT_PARAMFATITESC FAT
                JOIN FCT_ITESCOPOPROPOSTA ITESC
                  ON FAT.NRITESCOPOPROPOSTA = ITESC.NRITESCOPOPROPOSTA
                  AND FAT.NRTIPOORCAMENTO   = ITESC.NRTIPOORCAMENTO
                  AND FAT.NRORG             = ITESC.NRORG
                WHERE ITESC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO 
                  AND ITESC.NRORG           = :NRORG 
                  AND ITESC.CDSERVORCA IN(SELECT DISTINCT CDSERVORCA
                                          FROM FCT_ITNEGOCIACAOCONTRATO
                                          WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * busca os dados de vencimento dos itens de parcelamento
    **/
    public function getDadosVencimentoEscopo(array $params) {
        $sql = "SELECT FAT.QTDIASFATUVENC, 
                  FAT.NRDIACALCVENC, 
                  FAT.IDTPCALCVENC, 
                  FAT.IDMESCALCVENC,
                  FAT.QTDIASAPURACAO,
                  30 AS QTDIASCOMPETENCIA
                FROM FCT_CAPAPARAMFATESC FAT
                WHERE ITESC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO 
                  AND ITESC.NRORG           = :NRORG ";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * busca os dados da competência de uma negociação
    **/
    public function getDadosCompetencia(array $params) {
        $sql = "SELECT NRDIACOMPETINI,
                       NRDIACOMPETFIN,
                       IDMESCOMPETINI,
                       IDMESCOMPETFIN,
                       QTDIASAPURACAO,
                       IDTPCALCVENC,
                       NRDIACALCVENC,
                       IDMESCALCVENC,
                       QTDIASFATUVENC,
                       NRDIACOMPETFIN - NRDIACOMPETINI AS QTDIASCOMPETENCIA
                FROM FCT_COMPETENCIANEGOCONT
                WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                  AND NRORG                = :NRORG
                ORDER BY NRDIACOMPETINI";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os dados do servico
    **/
    public function getDadosServico(array $params) {
        $sql = "SELECT RELACSERV.CDSERVORCARELAC,
                  SUM(RELACSERV.QTMEDIDASERV) AS QTMEDIDASERV,
                  RELACSERV.CDTPMEDIDASERV
                FROM FCT_SERVICOORCA SERV
                JOIN FCT_RELACSERVICO RELACSERV
                  ON SERV.CDSERVORCA = RELACSERV.CDSERVORCA
                  AND SERV.NRORG     = RELACSERV.NRORG
                WHERE RELACSERV.CDSERVORCARELAC = :CDSERVORCA
                    AND RELACSERV.NRORG         = :NRORG 
                    AND SERV.IDELEMENTO IN(SELECT IDELEMENTO
                                          FROM (SELECT DISTINCT IDELEMENTO, IDELEMENTOPAI
                                                      FROM FCT_TRABSERVCONTRATO 
                                                      WHERE IDESCOLHIDO   = 'S'
                                                        AND IDATIVO       = 'S'
                                                        AND NRSEQCONTRATO = :NRSEQCONTRATO
                                                        AND NRORG         = :NRORG) 
                                          CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                                          START WITH IDELEMENTO       = :IDELEMENTO)
                GROUP BY CDSERVORCARELAC, CDTPMEDIDASERV";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca os dados da medida variável de um elemento
    **/
    public function getDadosTipoMedida($params) {
        $sql = "SELECT CDTPMEDIDAVARIAVEL,
                  QTMEDIDAVARIAVEL,
                  QTFRANQUIA
                FROM FCT_RELACMEDIDACONTRATO
                WHERE NRSEQCONTRATO  = :NRSEQCONTRATO
                  AND IDELEMENTORAIZ = :IDELEMENTORAIZ
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * busca as informações da tabela de preço ligada ao contrato
    **/
    public function getDadosVariacaoPreco(array $params) {
        $sql = "SELECT QTMEDIDAINIVARIACAO,
                       QTMEDIDAFIMVARIACAO,
                       VRPRECOVARIACAO,
                       IDMULTIPVARIACAO
                FROM FCT_VARPRECBASESERVCONT
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                  AND NRORG         = :NRORG
                  AND CDSERVORCA    = :CDSERVORCA
                  AND :QTMEDIDAORCA BETWEEN QTMEDIDAINIVARIACAO AND QTMEDIDAFIMVARIACAO
                  AND (CDTPMEDIDAVARIAVEL  = :CDTPMEDIDAVARIAVEL
                        OR CDTPMEDIDASERV  = :CDTPMEDIDAVARIAVEL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Insere um relacionamento entre uma parcela e seu contrato
    **/
    public function inserirRelacionamentoParcelaContrato(array $params) {
        $sql = "INSERT INTO FCT_RELACAGRUPCONTFAT (
                    NRAGRUPCONTFAT, NRFATURACONT, IDPARCAGRUP,
                    NRSEQCONTRATO,  NRORG,        IDATIVO,
                    DTINCLUSAO,     DTULTATU,     NRORGINCLUSAO,
                    CDOPERINCLUSAO, NRORGULTATU,  CDOPERULTATU,
                    NRMEDICAOFATURA
                ) VALUES (
                    :NRAGRUPCONTFAT, :NRFATURACONT, 'N',
                    :NRSEQCONTRATO,  :NRORG,        'S',
                    SYSDATE,         NULL,          :NRORG,
                    :CDOPERADOR,     NULL,          NULL,
                    NULL
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    } 
    /**
     * Retorna os itens da negociação de um contrato
    **/
    public function getItensNegociacaoContrato(array $params) {
        $sql = "SELECT ITNEG.NRITEMOCONTRATO,
                    ITNEG.VRTOTITEM,
                    ITNEG.VRPERCDESCITEM,
                    ITNEG.VRDESCITEM,
                    ITNEG.VRTOTITEMCOMDESC,
                    ITNEG.CDPRODFATURA
                FROM FCT_ITNEGOCIACAOCONTRATO ITNEG
                JOIN FCT_ITEMCONTRATO ITEM
                  ON ITNEG.NRITEMOCONTRATO = ITEM.NRITEMOCONTRATO
                  AND ITNEG.NRORG = ITEM.NRORG
                WHERE ITNEG.NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND ITNEG.NRORG               = :NRORG
                   AND ITEM.CDSERVORCA           = :CDSERVORCA
                   AND ITEM.IDELEMENTO           = :IDELEMENTO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Insere a parcela
    **/
    public function inserirParcela(array $params) {
        $sql = "INSERT INTO FCT_FATURACONT (
                    NRFATURACONT,     CDFILIALFATURA,  NRPARCFATURA,
                    CDTIPOOPERFATURA, CDSERIENF,	   IDTPFATURA,
                    CDEMPRESA,        CDINSCESTA,      CDINSCMUNI, 
                    IDSITUACAO,       DTINICOMPET,     DTFINCOMPET,
                    DTLIMITEAPU,      DTVENCIMENTO,    NRDIASVENCIMENTO,
                    QTDIASFATUVENC,   VRVENCIMENTO,    VRFATURADO,
                    VRFALTAFATURAR,   IDATIVO, 		   NRORG,
                    DTINCLUSAO,       NRORGINCLUSAO,   CDOPERINCLUSAO
                ) VALUES (
                    :NRFATURACONT,     :CDFILIALFATURA,  :NRPARCFATURA,
                    :CDTIPOOPERFATURA, :CDSERIENF,	     :IDTPFATURA,
                    :CDEMPRESA,        :CDINSCESTA,      :CDINSCMUNI, 
                    :IDSITUACAO,       :DTINICOMPET,     :DTFINCOMPET,
                    :DTLIMITEAPU,      :DTVENCIMENTO,    :NRDIASVENCIMENTO,
                    :QTDIASFATUVENC,   :VRVENCIMENTO,    0,
                    :VRFALTAFATURAR,   'S', 		     :NRORG,
                    SYSDATE,           :NRORG,           :CDOPERINCLUSAO
                )"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Insere os itens da parcela
    **/
    public function inserirItensParcela(array $params) {
        $sql = "INSERT INTO FCT_ITFATURACONT (
                    NRITFATURACONT,  NRFATURACONT,    NRSEQCONTRATO,
                    NRITEMOCONTRATO, CDPRODFATURA,    IDSITUACAO,
                    VRVENCIMENTO,    VRFATURADO,      VRFALTAFATURAR,
                    IDATIVO,		 NRORG,           DTINCLUSAO,     
                    NRORGINCLUSAO,   CDOPERINCLUSAO    
                ) VALUES (
                    :NRITFATURACONT,  :NRFATURACONT,    :NRSEQCONTRATO,
                    :NRITEMOCONTRATO, :CDPRODFATURA,    :IDSITUACAO,
                    :VRVENCIMENTO,    0,                :VRFALTAFATURAR,
                    'S',		 	  :NRORG,           SYSDATE,     
                    :NRORG,   		  :CDOPERADOR
                )"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}
