<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;
class Sm00001100ItemLancamentoPropostaFactory{
    protected $connection;
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    /**
     * retorna a quantidade de elementos distintos de uma árvore de elementos
    **/
    public function getCountElementos(array $params){
        $sql = "SELECT COUNT(IDELEMENTO) QTELEMENTOS, 
                  SUM(NVL(VRTOTMSLSAJUSTE, 0)) AS VRTOTSAJUSTE
                FROM (SELECT DISTINCT IDELEMENTO, VRTOTMSLSAJUSTE
                  FROM FCT_RELACSERVORCAMENTO
                  WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                     AND (IDELEMENTOPAI  = :IDELEMENTOPAI
                          OR IDELEMENTOPAI LIKE :IDELEMENTOPAI||'%')
                     AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                     AND CDSERVORCA      = :CDSERVORCA
                     AND NRORG           = :NRORG
                     AND IDESCOLHIDO     = 'S'
                     AND IDATIVO         = 'S')";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $quantidade = $std->fetchAll();
        return $quantidade[0];
    }
    /**
     * Exclui um item da tabela de agendamentos pendentes caso ele seja excluido na proposta
    **/
    public function excluiItemAgendPendente(array $params){
        $sql = "DELETE FROM FCT_AGENDPENDENTE
                WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND (NRITEMORCAMENTO = :NRITEMORCAMENTO
                        OR :NRITEMORCAMENTO IS NULL)
                    AND NRORG            = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Torna inativo os elementos da composição de serviço 
     * que o usuário pede para apagar
    **/
    public function inativarElementosProposta(array $params){
        $sql = "UPDATE FCT_RELACSERVORCAMENTO
                SET IDATIVO      = 'N',
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                    AND (IDELEMENTOPAI  = :IDELEMENTOPAI
                        OR IDELEMENTOPAI LIKE :IDELEMENTOPAI||'%')
                    AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                    AND CDSERVORCA      = :CDSERVORCA
                    AND NRORG           = :NRORG
                    AND IDESCOLHIDO     = 'S'
                    AND IDATIVO         = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retira do valor do item o valor do elemento inativado
    **/
    public function alterarValorItemPai(array $params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRTOTALITEMORCA = VRTOTALITEMORCA  - :VRELERETIRADO, 
                    VRUNITITEMORCA  = VRUNITITEMORCA - :VRELERETIRADO,
                    VRITEMSEMDESC   = VRITEMSEMDESC  - :VRELERETIRADO,
                    CDOPERULTATU    = :CDOPERULTATU
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                    AND CDSERVORCA   = :CDSERVORCA
                    AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Deleta os elementos de uma formação de preço excluída da proposta
    **/
    public function deletarItensFormacaoProp($params){
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITPRO
                WHERE NRSEQORCAMENTO        = :NRSEQORCAMENTO
                    AND NRRELACFORMACAOPROP = :NRRELACFORMACAOPROP
                    AND NRORG               = :NRORG
                    AND NRFORMPRECOSERVICO  = :NRFORMPRECOSERVICO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Deleta um elemento ou uma sub árvore de elementos de uma formação de preço da proposta
    **/
    public function deletarElementosFormacaoProp($params){
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITPRO
                WHERE NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV 
                                             FROM FCT_CALCFORMPRECSERV_ITPRO
                                             WHERE NRSEQORCAMENTO       = :NRSEQORCAMENTO
                                                AND NRRELACFORMACAOPROP = :NRRELACFORMACAOPROP
                                                AND NRORG               = :NRORG
                                                AND NRFORMPRECOSERVICO  = :NRFORMPRECOSERVICO
                                             CONNECT BY PRIOR NRCALCFORMPRECSERV = NRCALCFORMPRECSERVPAI
                                             START WITH NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Altera o valor no item pai do servico para utilizar valor total dos filhos 
    **/
    public function replicarAlteracoesItensFilhos(array $params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRTOTALITEMORCA = VRTOTALITEMORCA + :VRDIFERENCA,
                    VRUNITITEMORCA  = VRTOTALITEMORCA + :VRDIFERENCA,
                    VRITEMSEMDESC   = VRTOTALITEMORCA + :VRDIFERENCA
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                    AND CDSERVORCA   = :CDSERVORCA
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     *  Atualiza as quantidades dos elementos na tabela de serviço
    **/
    public function salvarNovasQuantidadesElemento(array $params){
        $sql = "UPDATE FCT_RELACSERVORCAMENTO 
                SET VRCUSTOSERV     = :VRCUSTOSERV,
                    VRTOTMENSALSERV = :VRTOTMENSALSERV,
                    VRTOTMSLSAJUSTE = :VRTOTMSLSAJUSTE,
                    QTSEGUNDA       = :QTSEGUNDA,
                    QTTERCA         = :QTTERCA,
                    QTQUARTA        = :QTQUARTA,
                    QTQUINTA        = :QTQUINTA,
                    QTSEXTA         = :QTSEXTA,
                    QTSABADO        = :QTSABADO,
                    QTDOMINGO       = :QTDOMINGO,
                    QTMENSAL        = :QTMENSAL,
                    HRENTRADA       = :HRENTRADA,
                    HRSAIDA         = :HRSAIDA,
                    NRORGULTATU     = :NRORGULTATU,
                    CDOPERULTATU    = :CDOPERULTATU
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                    AND CDSERVORCA   = :CDSERVORCA
                    AND IDELEMENTO IN(SELECT IDELEMENTO FROM FCT_RELACSERVORCAMENTO
                                        WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                            AND NRORG        = :NRORG
                                            AND CDSERVORCA   = :CDSERVORCA
                                        CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                                        START WITH IDELEMENTO = :IDELEMENTO)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}