<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00015004EquipeDeTrabalhoFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function saveRecurso(array $params){
        
        $sql = "INSERT INTO FCT_RECURSOSAGENDCONT (
                    NRRECURSOSAGENDCONT  , NRAGENDRECURSOCONT  , NRITEMOCONTRATO       ,
                    DTINIAGENDRECURSO    , DTFIMAGENDRECURSO   , IDTPRECURSO           ,
                    CDPRODUTO            , NRPARCNEPESSOA      , CDEMPRESA             ,
                    CDBEMATIVO           , NRRECURSOSUPERIOR   , IDMOTIVOAGENDAMENTO   ,
                    IDSITUARECGEND       , DSATIVIDADESAGEND   , HRPRIMEIRAENTRADA     ,
                    HRPRIMEIRASAIDA      , HRSEGUNDAENTRADA    , HRSEGUNDASAIDA        ,
                    NRTEMPOTOLERANCIA    , IDREPETDOMINGO      , IDREPETSEGUNDA        ,
                    IDREPETTERCA         , IDREPETQUARTA       , IDREPETQUINTA         ,
                    IDREPETSEXTA         , IDREPETSABADO       , NRORG                 ,
                    IDATIVO              , DTINCLUSAO          , NRORGINCLUSAO         ,
                    CDOPERINCLUSAO       , IDHORAEXTRA         , IDDESCANSOREMU        ,
                    IDALTERACAOEQUIPE    , NRRECURSOALTEQUIPE  , DSJUSTIFICAALTEQUIPE  ,
                    DTULTATU             , HRDESCANSO          , HRJORNADADIARIA       ,
                    IDESCALATRAB         , HRINTERVALO
                )
                VALUES (
                    :NRRECURSOSAGENDCONT , :NRAGENDRECURSOCONT , :NRITEMOCONTRATO      ,
                    :DTINIAGENDRECURSO   , :DTFIMAGENDRECURSO  , :IDTPRECURSO          ,
                    :CDPRODUTO           , :NRPARCNEPESSOA     , :CDEMPRESA            ,
                    :CDBEMATIVO          , :NRRECURSOSUPERIOR  , :IDMOTIVOAGENDAMENTO  ,
                    :IDSITUARECGEND      , :DSATIVIDADESAGEND  , :HRPRIMEIRAENTRADA    ,
                    :HRPRIMEIRASAIDA     , :HRSEGUNDAENTRADA   , :HRSEGUNDASAIDA       ,
                    :NRTEMPOTOLERANCIA   , :IDREPETDOMINGO     , :IDREPETSEGUNDA       ,
                    :IDREPETTERCA        , :IDREPETQUARTA      , :IDREPETQUINTA        ,
                    :IDREPETSEXTA        , :IDREPETSABADO      , :NRORG                ,
                    'S'                  , SYSDATE             , :NRORG                ,
                    :OPERADOR            , :IDHORAEXTRA        , :IDDESCANSOREMU       ,
                    :IDALTERACAOEQUIPE   , :NRRECURSOALTEQUIPE , :DSJUSTIFICAALTEQUIPE ,
                    :DTULTATU            , :HRDESCANSO         , :HRJORNADADIARIA      ,
                    :IDESCALATRAB        , :HRINTERVALO
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
    public function verificarDiaUtil(array $params) {
        $sql = "SELECT COUNT(*) AS EXISTE
                  FROM FERIFILI
                 WHERE DTFERIFILI = :DATA
                   AND NRORG      = :NRORG
                   AND CDFILIAL   = (SELECT CDFILIALORCA
                                       FROM FCT_CONTRATO
                                      WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                                        AND NRORG         = :NRORG)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getQuantidadeAgendamentosPendentes(array $params) {
        $sql = "SELECT QTPENDENCIA
                  FROM FCT_AGENDPENDENTE
                 WHERE NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                   AND NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function excluirAgendamentoPendente(array $params) {
        $sql = "DELETE FROM FCT_AGENDPENDENTE
                 WHERE NRSEQCONTRATO      = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO    = :NRITEMOCONTRATO
                   AND NRORG              = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function atualizarQuantidadeAgendamentoPendente(array $params) {
        $sql = "UPDATE FCT_AGENDPENDENTE
                   SET QTPENDENCIA        = QTPENDENCIA - 1,
                       NRAGENDRECURSOCONT = :NRAGENDRECURSOCONT,
                       DTINIAGENDAMENTO   = :DTINIAGENDAMENTO
                 WHERE NRSEQCONTRATO      = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO    = :NRITEMOCONTRATO
                   AND NRORG              = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getUrlIntegracaoHcm(array $params){
        $sql = "SELECT URLINTEGRACAOHCM 
                FROM FCT_PARAMETROGERAL
                WHERE NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();        
    }
    
    /**
     * Salva uma equipe de trabalho inserida manualmente pelo usuário
     * num recurso da equipe de trabalho
    **/
    public function saveEscalaTrabalho($params){
        $sql = "INSERT INTO FCT_ESCALARECURSOSAGENDCONT(NRRECURSOSAGENDCONT, NRESCALARECURSOSAGENDCONT, DTESCALAENTRADA, 
                    DTESCALASAIDA,       HRINICIAL,                 HRFINAL, 
                    NRORG,               IDATIVO,                   DTINCLUSAO,
                    DTULTATU,            NRORGINCLUSAO,             CDOPERINCLUSAO,
                    NRORGULTATU,         CDOPERULTATU,              HRINTERVALO)
                    VALUES(:NRRECURSOSAGENDCONT, :NRESCALARECURSOSAGENDCONT, :DTESCALAENTRADA, 
                        :DTESCALASAIDA,          :HRINICIAL,                 :HRFINAL, 
                        :NRORG,                  'S',                        SYSDATE,
                        NULL,                    :NRORGINCLUSAO,             :CDOPERINCLUSAO,
                        NULL,                    NULL,                        :HRINTERVALO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
   
    /**
     * Salva a edição de uma escala de trabalho inserida manualmente pelo usuário
    **/
    public function saveEdicaoEscalaTrabalho(array $params){
        $sql = "UPDATE FCT_ESCALARECURSOSAGENDCONT
                SET HRINICIAL = :HRINICIAL,
                HRFINAL       = :HRFINAL,
                HRINTERVALO   = :HRINTERVALO,
                DTESCALASAIDA = :DTESCALASAIDA
                WHERE DTESCALAENTRADA = :DTESCALAENTRADA
                    AND NRORG = :NRORG
                    AND NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
   
    /**
     * Salva a edição de uma escala de trabalho inserida manualmente pelo usuário
    **/
    public function desalocaEscalaTrabalho(array $params){
        $sql = "DELETE FROM FCT_ESCALARECURSOSAGENDCONT
                WHERE DTESCALAENTRADA >= :DTESCALAENTRADA
                    AND NRORG = :NRORG
                    AND NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
   
    /**
     * deleta uma escala de trabalho inserida manualmente pelo usuário
    **/
    public function deletarEscala(array $params, $nrrecurso){
        $sql = "DELETE FROM FCT_ESCALARECURSOSAGENDCONT
                WHERE NRORG = :NRORG
                    AND NRRECURSOSAGENDCONT IN(".$nrrecurso.")";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os dados de uma escala de trabalho a partir de 
     * determinada data
    **/
    public function getEscalaTrabalho(array $params){
        $sql = "SELECT *
                FROM FCT_ESCALARECURSOSAGENDCONT ESCALA
                LEFT JOIN FCT_RECURSOSAGENDCONT RECU ON ESCALA.NRRECURSOSAGENDCONT = RECU.NRRECURSOSAGENDCONT
                WHERE ESCALA.NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT
                    AND ESCALA.NRORG             = :NRORG
                    AND ((DTESCALAENTRADA BETWEEN TO_DATE(:DTESCALAENTRADA)
                            AND TO_DATE(:DTESCALASAIDA))
                        OR (DTESCALAENTRADA >= TO_DATE(:DTESCALAENTRADA) 
                            AND :DTESCALASAIDA IS NULL))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Valida o perído de alocação para verificar se o recurso escolhido
     * esta disponível naquela
    **/
    public function verificaDisponibilidadeRec(array $params, $diadasemana, $chave){
        $sql = "SELECT  COUNT(*) EXISTE
                FROM FCT_RECURSOSAGENDCONT EQUIPE
                LEFT JOIN FCT_ESCALARECURSOSAGENDCONT ESCALA
                    ON EQUIPE.NRRECURSOSAGENDCONT = ESCALA.NRRECURSOSAGENDCONT
                    AND EQUIPE.NRORG              = ESCALA.NRORG
                INNER JOIN FCT_VIEW_RECURSOS VIEWREC 
                    ON NVL(TO_CHAR(EQUIPE.NRPARCNEPESSOA), NVL(EQUIPE.CDBEMATIVO, EQUIPE.CDPRODUTO)) = VIEWREC.CHAVE 
                WHERE CHAVE IN ('".$chave."')
                    AND EQUIPE.NRORG              = :NRORG
                    AND EQUIPE.IDSITUARECGEND     = 1
                    AND ((EQUIPE.IDESCALATRAB = 'M' 
                            AND ESCALA.DTESCALAENTRADA = TO_DATE(:DTENTRADA, 'DD/MM/YY')
                            AND HRINICIAL IS NOT NULL)
                        OR ((EQUIPE.IDESCALATRAB = 'N' OR EQUIPE.IDESCALATRAB IS NULL) 
                            AND TO_DATE(:DTENTRADA, 'DD/MM/YY') BETWEEN EQUIPE.DTINIAGENDRECURSO AND EQUIPE.DTFIMAGENDRECURSO
                            AND ".$diadasemana." = 'S'))
                    AND ((EQUIPE.NRRECURSOSAGENDCONT <> :NRRECURSOSAGENDCONT)
                        OR (:NRRECURSOSAGENDCONT IS NULL))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * retorna o número do contrato ligado ao agendamento
     * caso a tela não envie-o na requisição 
    * */
    public function getNrContrato(array $params){
        $sql = "SELECT DISTINCT NRSEQCONTRATO
                FROM FCT_RECURSOSAGENDCONT EQUIPE 
                INNER JOIN FCT_AGENDRECURSOCONT AGEND
                    ON EQUIPE.NRAGENDRECURSOCONT = AGEND.NRAGENDRECURSOCONT
                    AND EQUIPE.NRORG = AGEND.NRORG
                WHERE EQUIPE.NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT
                    AND EQUIPE.NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Retorna os dias não trabalhados 
     * na escala de trabalho ligada ao agendamento
    **/
    public function getDiasNaotrabalhadosEscala(array $params){
        $sql = "SELECT *
                FROM FCT_ESCALARECURSOSAGENDCONT
                WHERE NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT
                    AND NRORG = :NRORG
                    AND DTESCALAENTRADA BETWEEN :DTINIAGENDAMENTO AND :DTFIMAGENDAMENTO
                    AND HRINICIAL IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Valida se o item já possui um agendamento para o dia e 
     * o período definido
    **/
    public function getValidacaoItemPeriodo(array $params, $diadasemana) {
        $sql = "SELECT COUNT(*) QTALOCADO, 
                    NVL(QTITEMCONTRATO, 0) AS QTITEMCONTRATO
                FROM FCT_RECURSOSAGENDCONT EQUIPE
                JOIN FCT_ITEMCONTRATO ITEMCONT
                  ON EQUIPE.NRITEMOCONTRATO = ITEMCONT.NRITEMOCONTRATO
                  AND EQUIPE.NRORG          = ITEMCONT.NRORG
                WHERE EQUIPE.IDSITUARECGEND    = '1'
                    AND EQUIPE.NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND EQUIPE.NRORG           = :NRORG
                    AND (TO_DATE(:DTATUAL) BETWEEN TO_DATE(TO_CHAR(EQUIPE.DTINIAGENDRECURSO, 'DD/MM/YY'), 'DD/MM/YY') 
                            AND TO_DATE(TO_CHAR(EQUIPE.DTFIMAGENDRECURSO, 'DD/MM/YY'), 'DD/MM/YY'))
                    $diadasemana
                GROUP BY QTITEMCONTRATO"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0];
    }
    /**
     * 
    **/
    public function validarPeriodosAlocacao(array $params) {
        $sql = "SELECT COUNT(*) QTALOCADO,
                    QTITEMCONTRATO,
                    IDREPETDOMINGO,
                    IDREPETSEGUNDA,
                    IDREPETTERCA,
                    IDREPETQUARTA,
                    IDREPETQUINTA,
                    IDREPETSEXTA,
                    IDREPETSABADO
                FROM (SELECT DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETDOMINGO) AS IDREPETDOMINGO,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETSEGUNDA) AS IDREPETSEGUNDA,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETTERCA) AS IDREPETTERCA,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETQUARTA) AS IDREPETQUARTA,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETQUINTA) AS IDREPETQUINTA,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETSEXTA) AS IDREPETSEXTA,
                        DECODE(EQUIPE.IDDESCANSOREMU, 'S', 'S', EQUIPE.IDREPETSABADO) AS IDREPETSABADO,
                        ITEMCONT.QTITEMCONTRATO
                      FROM FCT_RECURSOSAGENDCONT EQUIPE
                      JOIN FCT_ITEMCONTRATO ITEMCONT
                        ON EQUIPE.NRITEMOCONTRATO = ITEMCONT.NRITEMOCONTRATO
                        AND EQUIPE.NRORG          = ITEMCONT.NRORG
                      WHERE EQUIPE.NRITEMOCONTRATO    = :NRITEMOCONTRATO
                        AND EQUIPE.NRORG              = :NRORG
                        AND EQUIPE.IDSITUARECGEND     = '1'
                        AND EQUIPE.NRAGENDRECURSOCONT = :NRAGENDRECURSOCONT
                        AND (:DTINIAGENDRECURSO BETWEEN DTINIAGENDRECURSO AND DTFIMAGENDRECURSO
                             OR :DTFIMAGENDRECURSO BETWEEN DTINIAGENDRECURSO AND DTFIMAGENDRECURSO
                             OR DTINIAGENDRECURSO BETWEEN :DTINIAGENDRECURSO AND :DTFIMAGENDRECURSO
                             OR DTFIMAGENDRECURSO BETWEEN :DTINIAGENDRECURSO AND :DTFIMAGENDRECURSO))
                GROUP BY  QTITEMCONTRATO,
                    IDREPETDOMINGO,
                    IDREPETSEGUNDA,
                    IDREPETTERCA,
                    IDREPETQUARTA,
                    IDREPETQUINTA,
                    IDREPETSEXTA,
                    IDREPETSABADO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}